#include "SDL.h"
#include "SDL_ttf.h"
#include <stdio.h>
#include <string.h>

#include "auxiliar.h"
#include "collision.h"


Uint8 bit_mask[8]={0x01,0x02,0x04,0x08,0x10,0x20,0x40,0x80};
SDL_Rect _ua;

int memand(Uint8 *s1, Uint8 *s2, int shift1, int shift2, int N);
int MOG2_bbcheck(MOG2CollisionData *cd1,Sint16 x1,Sint16 y1, MOG2CollisionData *cd2,Sint16 x2,Sint16 y2);
int _MOG2_collision_check(MOG2CollisionData *cd1,Sint16 x1,Sint16 y1, MOG2CollisionData *cd2,Sint16 x2,Sint16 y2);


MOG2CollisionData *MOG2_make_cmap(SDL_Surface *img)
{
	MOG2CollisionData *cdata;
	Uint8 *map;
	Sint16 x,y;
	Sint32 offs;
	int i;
	
	cdata=new MOG2CollisionData;
	cdata->w=img->w; cdata->h=img->h;
	offs=(img->w*img->h)/8;
	cdata->map=new Uint8[offs+2];
	memset(cdata->map,0x00,offs+2);
	
	map=cdata->map;
		
	i=0;
	for(y=0; y < img->h; y++){
		for(x=0; x < img->w; x++){
			if(i>7){i=0;map++;}
			if(getpixel(img, Sint16(x),Sint16(y))!=img->format->colorkey){
				*map=*map|bit_mask[i];	
			}
			i++;
		}	
	}
	return cdata;
}


int MOG2_bbcheck(MOG2CollisionData *cd1,Sint16 x1,Sint16 y1, MOG2CollisionData *cd2,Sint16 x2,Sint16 y2)
{
	Sint16 w1=cd1->w;
	Sint16 h1=cd1->h;
	Sint16 w2=cd2->w;
	Sint16 h2=cd2->h;

	if(x1 < x2){
		if(x1+w1 > x2){
			if(y1 < y2){
				if(y1+h1 > y2){
					_ua.x=x2;
					_ua.y=y2;
					return 1;
				}
			}
			else{
				if(y2+h2 > y1){
					_ua.x=x2;
					_ua.y=y1;
					return 1;
				}
			}
		}	
	}
	else{
		if(x2+w2 > x1){
			if(y2 < y1){
				if(y2+h2 > y1){
				  _ua.x=x1;
					_ua.y=y1;
					return 1;
				}
			}
			else{
				if(y1+h1 > y2){
				  _ua.x=x1;
					_ua.y=y2;
					return 1;
				}
			}
		}	
	}
	
	
	return 0;
}



int memand(Uint8 *s1, Uint8 *s2, int shift1, int shift2, int N)
{
	int b,i1=shift1,i2=shift2;

  for(b=0; b<N; b++){
  	if(i1>7){i1=0;s1++;}
  	if(i2>7){i2=0;s2++;}
  	if( (*s1&bit_mask[i1]) && (*s2&bit_mask[i2]) ) return b+1;
  	i1++; i2++;	
  }
	return 0;
}


int _MOG2_collision_check(MOG2CollisionData *cd1,Sint16 x1,Sint16 y1, MOG2CollisionData *cd2,Sint16 x2,Sint16 y2)
{
	if(cd1->map==0 || cd2->map==0)
		return 0;

  Sint16 w1=cd1->w;
	Sint16 h1=cd1->h;
	Sint16 w2=cd2->w;
	Sint16 h2=cd2->h;
	
	//masks
	
	Sint32 x1o=0,x2o=0,y1o=0,y2o=0,offs;  //offsets
	int i1=0,i2=0;
	
  Uint8 *map1=cd1->map;
	Uint8 *map2=cd2->map;
	
	//Calculate correct starting point
	if(_ua.x==x2 && _ua.y==y2){
		x1o=x2-x1;
		y1o=y2-y1;
		
		offs=w1*y1o+x1o;	
		map1+=offs/8;
		i1=offs%8;	
	}
	else if(_ua.x==x2 && _ua.y==y1){
		x1o=x2-x1;
		y2o=y1-y2;
			
		map1+=x1o/8;
		i1=x1o%8;
		
		offs=w2*y2o;
		map2+=offs/8;
		i2=offs%8;	
	}
	else if(_ua.x==x1 && _ua.y==y1){
		x2o=x1-x2;
		y2o=y1-y2;
		
		offs=w2*y2o+x2o;	
		map2+=offs/8;
		i2=offs%8;	
	}
	else if(_ua.x==x1 && _ua.y==y2){
		x2o=x1-x2;
		y1o=y2-y1;	
		
		offs=w1*y1o;
		map1+=offs/8;
		i1=offs%8;
		
		map2+=x2o/8;
		i2=x2o%8;	
	}
	else
		return 0;
		
	Sint16 y;

	Sint16 lenght;
	
	if(x1+w1 < x2+w2)
		lenght=w1-x1o;	
	else
		lenght=w2-x2o;
	
	//AND(map1,map2)
	for(y=_ua.y; y<=y1+h1 && y<=y2+h2; y++){
	
		offs=memand(map1,map2,i1,i2,lenght);
		if(offs){
			return 1;
		}
		
		//goto the new line
		offs=(y-y1)*w1+x1o;
		map1=cd1->map; //reset pointer
		map1+=offs/8;
		i1=offs%8;
		
    offs=(y-y2)*w2+x2o;
    map2=cd2->map; //reset pointer
		map2+=offs/8;
		i2=offs%8;
	}
	
	
	return 0;
}


int MOG2_collision_check(MOG2CollisionData *cd1,Sint16 x1,Sint16 y1, MOG2CollisionData *cd2,Sint16 x2,Sint16 y2)
{
	if(!MOG2_bbcheck(cd1,x1,y1, cd2,x2,y2))
		return 0;
		
	if(cd1->map==0 || cd2->map==0)
		return 1;	
	
	return _MOG2_collision_check(cd1,x1,y1, cd2,x2,y2); 	
}



void MOG2_destroy_cmap(MOG2CollisionData *cd)
{
	if(cd->map!=0)
		delete [] cd->map;
		
	delete cd;
}



