#include "assert.h"

#include "stdio.h"
#include "string.h"
#include "math.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_rotozoom.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "variables.h"
#include "mog2.h"
#include "tiles.h"

#include "auxiliar.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif



PLACED_TILE::PLACED_TILE(void)
{
	x=0;
	y=0;
	tile=0;
} /* PLACED_TILE::PLACED_TILE */ 


PLACED_TILE::PLACED_TILE(long int nx,long int ny,TILE *t)
{
	x=nx;
	y=ny;
	tile=t;
} /* PLACED_TILE::PLACED_TILE */ 


PLACED_TILE::PLACED_TILE(PLACED_TILE &t)
{
	x=t.x;
	y=t.y;
	tile=t.tile;
} /* PLACED_TILE::PLACED_TILE */ 


PLACED_TILE::~PLACED_TILE(void)
{
} /* PLACED_TILE::~PLACED_TILE */ 


bool PLACED_TILE::save(FILE *fp,MOG2 *game)
{
	int j,t=-1;
	bool found=false;

	for(j=0;!found && j<TILECATEGORIES;j++) {
		t=game->tiles[j].SearchObjRef(tile);
		if (t>=0) found=true;
	} /* for */ 
	assert(found);
	fprintf(fp,"%i %i %i %i\n",int(x),int(y),j-1,t);
	return true;
} /* PLACED_TILE::save */ 


bool PLACED_TILE::load(FILE *fp,MOG2 *game)
{
	int tmp,tmp2;
	int xtmp,ytmp;

	if (4!=fscanf(fp,"%i %i %i %i",&xtmp,&ytmp,&tmp,&tmp2)) return false;
	x=xtmp;
	y=ytmp;

	tile=game->tiles[tmp][tmp2];

	assert(tile!=0);
	return true;
} /* PLACED_TILE::load */ 


void PLACED_TILE::draw(SDL_Surface *sfc)
{
	if (tile!=0) tile->draw(x,y,sfc);
} /* PLACED_TILE::draw */ 



void PLACED_TILE::draw(long int sx,long int sy,SDL_Surface *sfc)
{
	if (tile!=0) tile->draw(sx+x,sy+y,sfc);
} /* PLACED_TILE::draw */ 


void PLACED_TILE::draw_collision_mask(long int sx,long int sy,SDL_Surface *sfc)
{
	if (tile!=0) tile->draw_collision_mask(sx+x,sy+y,sfc);
} /* PLACED_TILE::draw_collision_mask */ 


void PLACED_TILE::draw_scaled(long int sx,long int sy,SDL_Surface *dest,float scale)
{
	if (tile!=0) tile->draw_transform(int(sx+x*scale),int(sy+y*scale),dest,scale,0);
} /* PLACED_TILE::draw_Scaled */ 


void PLACED_TILE::draw(long int sx,long int sy,SDL_Surface *sfc,SDL_Rect vp)
{
	if (tile!=0 &&
		sx+x+tile->get_dx()>vp.x &&
		sy+y+tile->get_dy()>vp.y &&
		sx+x<vp.x+vp.w &&
		sy+y<vp.y+vp.h) tile->draw(sx+x,sy+y,sfc);
} /* PLACED_TILE::draw */ 


void PLACED_TILE::draw_collision_mask(long int sx,long int sy,SDL_Surface *sfc,SDL_Rect vp)
{
	if (tile!=0 &&
		sx+x+tile->get_dx() &&
		sy+y+tile->get_dy() &&
		sx+x<vp.x+vp.w &&
		sy+y<vp.y+vp.h) tile->draw_collision_mask(sx+x,sy+y,sfc);
} /* PLACED_TILE::draw_collision_mask */ 
