#include "string.h"
#include "stdlib.h"

#include "SDL.h"
#include "SDL_ttf.h"

#include "BList.h"
#include "auxiliar.h"

#include "variables.h"
#include "mog2.h"
#include "tiles.h"

#include "BWindow.h"
#include "BActiveWindow.h"
#include "BStringEdDlg.h"
#include "BIntegerEdDlg.h"
#include "BChoiceDlg.h"
#include "BVChoiceDlg.h"

#include "GrabGrabberWnd.h"
#include "interfaces.h"

#include "editor.h"

#ifdef KITSCHY_DEBUG_MEMORY 
#include "debug_memorymanager.h"
#endif

const int min_map_size=64;

GrabGrabberWnd::GrabGrabberWnd(int x,int y,int dx,int dy,EDITOR *e,TTF_Font *font) : BWindow(x,y,dx,dy,font) 
{
	editor=e;
	init_x=init_y=0;

} /* GrabGrabberWnd::GrabGrabberWnd */ 


GrabGrabberWnd::~GrabGrabberWnd()
{
} /* GrabGrabberWnd::~GrabGrabberWnd */ 


bool GrabGrabberWnd::cycle(int stx,int sty)
{
	int values[6]={1,2,4,8,16,32};
	int grid_x,grid_y;

	BWindow::cycle(stx,sty);

	if (editor->grabv_grid>=0) {
		grid_x=values[editor->grabv_grid];
		grid_y=values[editor->grabv_grid];
	} else {
		grid_x=editor->grabv_grid_x;
		grid_y=editor->grabv_grid_y;
	} /* if */ 
	pointer_x=(local_mouse_x-1);
	pointer_y=(local_mouse_y-1);
	if (pointer_x>=0 && pointer_y>=0 &&
		editor->grabv_bitmap!=0 &&
		pointer_x<=editor->grabv_bitmap->w &&
		pointer_y<=editor->grabv_bitmap->h) {
		pointer_x+=grid_x/2;
		pointer_y+=grid_y/2;
		pointer_x/=grid_x;
		pointer_y/=grid_y;
		pointer_x*=grid_x;
		pointer_y*=grid_y;
		editor->set_all_dirty();
	} else {
		if (pointer_x!=-1) {
			pointer_x=-1;
			pointer_y=-1;
			editor->set_all_dirty();
		} /* if */ 
	} /* if */ 

/*
	if (pointer_x<0) pointer_x=0;
	if (pointer_y<0) pointer_y=0;

	if (editor->grabv_bitmap!=0) {
		if (pointer_x>=editor->grabv_bitmap->w) pointer_x=editor->grabv_bitmap->w-1;
		if (pointer_y>=editor->grabv_bitmap->h) pointer_y=editor->grabv_bitmap->h-1;
	} /* if */ 

	if (mouse_button&SDL_BUTTON(1) &&
		!(old_mouse_button&SDL_BUTTON(1)) &&
		pointer_x>=0 &&
		pointer_y>=0 &&
		pointer_x<=editor->grabv_bitmap->w &&
		pointer_y<=editor->grabv_bitmap->h) {
		switch(editor->grabv_state) {
		case 0:
			init_x=pointer_x;
			init_y=pointer_y;
			editor->grabv_state=1;
			break;
		case 1:/* Grab Bitmap: */ 
			{
				List<TILE_SOURCE> l;
				TILE_SOURCE *b=0,*actb=0;
				bool found=false;

				l.Instance(editor->game->tile_sources);
				l.Rewind();
				while(!found && l.Iterate(b)) {
					if (b->cmp(editor->grabv_bitmap_file)) {
						found=true;
						actb=b;
					} /* if */ 
				} /* while */ 

				if (!found) {
					/* Add TILE_SOURCE to game: */ 
					actb=new TILE_SOURCE(editor->grabv_bitmap_file);

					editor->game->tile_sources.Add(actb);
				} /* if */ 

				/* Create a new TILE with the screen coordinates and the TILE_SOURCE: */ 
				{
					TILE *mb;
					int x1,y1,x2,y2;
					x1=init_x;
					x2=pointer_x;
					if (x1>x2) {
						x1=pointer_x;
						x2=init_x;
					} /* if */ 
					y1=init_y;
					y2=pointer_y;
					if (y1>y2) {
						y1=pointer_y;
						y2=init_y;
					} /* if */ 

					mb=new TILE(x1,y1,x2-x1,y2-y1,actb->sfc,actb->fname);
					if (editor->ov_regrabbing==0) {
						editor->game->tiles[editor->ov_bitmap_category].Add(mb);
					} else {
						editor->ov_regrabbing->free();
						editor->ov_regrabbing->instance(mb);
						mb->clear();
						delete mb;
						mb=editor->ov_regrabbing;
					} /* if */ 


					{
						int res;
						BVChoiceDlg *dlg2;

						dlg2=new BVChoiceDlg(-1,-1,0,0,"Choose Mask:","No Mask\nAutomatic Mask\nFrom Bitmap\n",&res,font);
						dlg2->Execute();
						delete dlg2;

						if (res==1) {
							SDL_SetAlpha(mb->orig,0,SDL_ALPHA_OPAQUE);
							mb->mask_type=0;
						} /* if */ 

						if (res==2) {
							SDL_SetAlpha(mb->orig,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
							surface_automatic_alpha(mb->orig);
							mb->mask_type=1;
						} /* if */ 

						if (res==3) {
							SDL_SetAlpha(mb->orig,SDL_SRCALPHA,SDL_ALPHA_OPAQUE);
							surface_mask_from_bitmap(mb->orig,actb->sfc,mb->r.x+mb->r.w,mb->r.y);
							found=true;
							mb->mask_type=2;
						} /* if */ 
					}
				}
			}
			editor->grabv_state=0;
			break;
		} /* switch */ 
	} /* if */ 

	return true;
} /* GrabGrabberWnd::cycle */ 


void GrabGrabberWnd::contens_size(int *dx,int *dy)
{
	int max_x=0;
	int max_y=0;
	
	BWindow::contens_size(dx,dy);

	if (editor->grabv_bitmap!=0) {
		max_x=editor->grabv_bitmap->w+2;
		max_y=editor->grabv_bitmap->h+2;
	} /* if */ 

	if (max_x>*dx) *dx=max_x;
	if (max_y>*dy) *dy=max_y;
} /* GrabGrabberWnd::contens_size */ 


void GrabGrabberWnd::draw_contens(SDL_Surface *sfc,int stx,int sty,int dx,int dy)
{
	BWindow::draw_contens(sfc,stx,sty,dx,dy);

	if (editor->grabv_bitmap!=0) {
		SDL_Rect s,d;

		s.x=0;
		s.y=0;
		s.w=editor->grabv_bitmap->w;
		s.h=editor->grabv_bitmap->h;
		d.x=1-stx;
		d.y=1-sty;
		d.w=editor->grabv_bitmap->w;
		d.h=editor->grabv_bitmap->h;
		SDL_BlitSurface(editor->grabv_bitmap,&s,sfc,&d);

		draw_rectangle(sfc,0-stx,0-sty,editor->grabv_bitmap->w+1,editor->grabv_bitmap->h+1,0x00ffffff);

		if (pointer_x>=0 && pointer_x<=editor->grabv_bitmap->w &&
			pointer_y>=0 && pointer_y<=editor->grabv_bitmap->h) {

			switch(editor->grabv_state) {
			case 0:draw_line(sfc,pointer_x+1-stx,1-sty,pointer_x+1-stx,editor->grabv_bitmap->h+1-sty,0x00ffffff);
				   draw_line(sfc,1-stx,pointer_y+1-sty,editor->grabv_bitmap->w+1-stx,pointer_y+1-sty,0x00ffffff);
				break;
			case 1:{
					int x1,y1,x2,y2;
					x1=init_x;
					x2=pointer_x;
					if (x1>x2) {
						x1=pointer_x;
						x2=init_x;
					} /* if */ 
					y1=init_y;
					y2=pointer_y;
					if (y1>y2) {
						y1=pointer_y;
						y2=init_y;
					} /* if */ 
					draw_rectangle(sfc,x1+1-stx,y1+1-sty,(x2-x1),(y2-y1),0x00ffffff);
				   }
				break;
			} /* switch */ 
		} /* if */ 

	} /* if */ 

} /* GrabGrabberWnd::draw_contens */ 



bool GrabGrabberWnd::button_pressed(int ID)
{

	return true;
} /* GrabGrabberWnd::button_pressed */ 
